__docstring__ = """The if plugin is for evaluating conditions and running commands based on them
USAGE: 'if [condition] ([command][&[command]] [else if [condition] ([command][&[command]]] [else [condition] ([command][&[command]]]'
Where [condition] is one of the following:
    errorlevel [level]
    string=%variable%'"""


import os, subprocess, config, varies, random, common
from errordb import errnos

def my_import(name):
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod
    

    
def run(line, startwd):
    commands = os.listdir(startwd + '/plugins')
    name = line[0].replace('.exe', '')
    if name + '.py' in commands:
        
        plugin = my_import('plugins.' + name)
        if name not in config.no_reload: reload(plugin)
        if 'main' in dir(plugin):
            out = plugin.main(line, startwd, config) # in a plugin, you must accept all of these arguments in your "main" function
            if type(out) == str: 
                if out.startswith('!goto: '): return out
            elif type(out) == type(my_import): 
                try: out() 
                except: print 'Unexpected fatal error in plugins main function'
            elif type(out) == int: errorlevel = out
            elif type(out) == tuple and len(out) == 2: # it can also return a two-tuple of (string, errno)
                errorlevel = out[1]
            elif type(out) == type(None): pass
            else: print 'DOSprompt plugin "%s" returned an invalid type: "%s". Please report this at https://launchpad.net/dosprompt/' % (name, str(type(out)))
    elif name in commands:
        try: subprocess.call([startwd + "/plugins/" + line[0].replace('.exe', '')] + line[1:])
        except OSError, e: 
            if e.errno in errnos.keys():
                print "Unix error " + str(e.errno) + ':', errnos.get(e.errno, "Unknown error")
            else: print "No such file or directory" # TODO: change this to a more DOSey message
        except KeyboardInterrupt: pass
    else:
        try: 
            subprocess.call([line[0].replace('.exe', '')] + line[1:])
        except OSError, e: 
            if e.errno in errnos.keys():
                print "Unix error " + str(e.errno) + ':', errnos.get(e.errno, "Unknown error")
            else: print "No such file or directory" # TODO: change this to a more DOSey message
        except KeyboardInterrupt: pass
        
   
def evaluate(terms, cd, conf):
    err = varies.get('errorlevel', 0, int)
    
    try: unix_path = common.unix_path(os.path.normpath(terms[1]), True)
    except OSError: unix_path = common.unix_path(os.path.normpath(terms[1]))
    
    if terms[0].lower() == 'exist' and os.path.exists(unix_path) and terms[2].startswith('('):
        rest = ' '.join(terms).split('(')[1].split(')')[0]
        for cmd in rest.split("&"):
            out = run(cmd.split(), cd)
        if out: return out
        return True
    elif len(terms[0].split("=")) > 1:
        if terms[0].split("=")[0] == terms[0].split("=")[1]:
            rest = ' '.join(terms).split('(')[1].split(')')[0]
            for cmd in rest.split("&"):
                out = run(cmd.split(), cd)
            if out: return out
            return True  
            
    elif terms[0].lower() == 'errorlevel' and str(err) == terms[1]:
        rest = ' '.join(terms).split('(')[1].split(')')[0]
        for cmd in rest.split("&"):
            out = run(cmd.split(), cd)
        if out: return out
        return True  
        
    #if nothing has been returned by now, just return false
    return False
        
def main(line, cd, conf): 
    if len(line) > 2:
        done = evaluate(line[1:], cd, conf)
        next = ' '.join(line).split(')')
        if len(next) > 1 and not done: # looks like there might be else and/or else if
        
            for x in next:
                if done: break
                if x.strip().startswith("else if") and not done:
                    done = evaluate(x.strip().split("else if ")[1].split(), cd, conf)

                elif x.strip().startswith("else (") and not x.strip().startswith("else if") and not done:
                    for cmd in x.strip().split("else (")[1].split('&'):
                        done = run(cmd.split(), cd)
                    if random.randint(0, 100) == 42: print 'The meaning of life, the universe, and everything is... BACON!!!!'
        done = str(done)
        if done.startswith('!'): return done


                    



